package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

import java.util.Collection;
import java.util.LinkedList;

public class ConnectionImplementation implements Connection {

	private Link link;
	private LinkedList<Port> participants;
	private LinkedList<Port> removedParticipants;
	private Protocol protocol;
	private double packetLossRate;
	private byte status;
	private boolean changed = false;
	private String name;
	
	public ConnectionImplementation(Link l, Protocol p) {
		link = l;
		participants=new LinkedList<Port>();
		removedParticipants = new LinkedList<Port>();
		this.protocol = p;
		status =Connection.ACTIVE;
		if(p==null)
			name = "unspecified";
		else
			name = p.getName()+"-connection";
	}
	
	@Override
	public Link getLink() {
		return link;
	}

	@Override
	public Port getSource() {
		if(participants.size()==0)return null;
		return participants.getFirst();
	}

	@Override
	public Collection<Port> getParticipants() {
		LinkedList<Port> out = new LinkedList<Port>();
		out.addAll(participants);
		out.addAll(removedParticipants);
		return out;
	}

	@Override
	public boolean removeSmartDevice(Port sd) {
		boolean removed = participants.remove(sd);
		if(removed)
			removedParticipants.add(sd);
		if(participants.size()<2)setStatus(Connection.TERMINATED);
		return removed;
	}

	@Override
	public boolean addSmartDevice(Port sd) {
		return participants.add(sd);
	}

	@Override
	public Collection<Packet> simulateTimeInterval(long startTime, long duration) {
		LinkedList<Packet> returnPackets=new LinkedList<Packet>();
		returnPackets.addAll(getTerminationPackages(startTime));
		for(Port p:participants){
			if(p.getLastTrigger()+p.getTriggerInterval()<startTime && p.getStatus()==Port.SENDING){
				returnPackets.addAll(protocol.generateNextPakets(p, (long) (startTime+p.getJitter()*Math.random()/2),Math.random()<packetLossRate));
			}
			while(p.getLastTrigger()+p.getTriggerInterval()<startTime+duration &&p.getStatus()==Port.SENDING)
				returnPackets.addAll(protocol.generateNextPakets(p, (long) (p.getLastTrigger()+p.getTriggerInterval()+p.getJitter()*(Math.random()-0.5)),Math.random()<packetLossRate));
		}
		returnPackets.sort((a,b)->(Long.compare(a.getTimestamp(),b.getTimestamp())));
		return returnPackets;
	}

	@Override
	public Collection<Packet> getTerminationPackages(long startTime) {
		changed=!removedParticipants.isEmpty();
		removedParticipants.clear();
		if(status==TERMINATED)status = DONE;
		return protocol.generateNextPakets(null, startTime, false);
	}

	@Override
	public Protocol getProtocol() {
		return protocol;
	}

	@Override
	public boolean setProtocol(Protocol protocol) {
		this.protocol = protocol;
		return true;
	}

	@Override
	public byte getStatus() {
		return status;
	}

	@Override
	public void setStatus(byte status) {
		this.status = status;
	}

	@Override
	public void setPacketLossProbability(double lossPercentage) {
		packetLossRate=lossPercentage;
	}

	@Override
	public double getPacketLossProbability() {
		return packetLossRate;
	}

	@Override
	public boolean getStatusChanged() {
		return changed;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public void setName(String name) {
		this.name = name;
	}

}
