package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

import java.util.Collection;

/**
 * Physical Link medium for SmartDevices, which connects two or more devices and
 * allows communication between them
 *
 * @author Andreas T. Meyer-Berg
 */
public interface Link {
	/**
	 * @return the name
	 */
	public String getName();

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(String name);

	/**
	 * @return the devices
	 */
	public Collection<SmartDevice> getDevices();

	/**
	 * @param device
	 *            the devices to add
	 */
	public void addDevice(SmartDevice device);

	/**
	 * @param device
	 *            the devices to remove
	 */
	public void removeDevice(SmartDevice device);
	
	/**
	 * @return the connections
	 */
	public Collection<Connection> getConnections();

	/**
	 * @param connection
	 *            the connection to add
	 */
	public void addConnection(Connection connection);
	
	/**
	 * @param connection
	 *            the connection to remove
	 */
	public void removeConnection(Connection connection);

	/**
	 * Simulates an interval starting at startTime for a given duration
	 * 
	 * @param startTime
	 *            Time the simulation interval starts in
	 *            System.currentTimeMillis() time
	 * @param duration
	 *            Duration of the simulation interval in milliseconds
	 */
	public void simulateTimeInterval(long startTime, long duration);

	/**
	 * Returns all packets which where sent during the last Simulation time step
	 * 
	 * @return packets, which were sent during the last time step
	 */
	public Collection<Packet> getPackets();
	
	/**
	 * Returns true if one or more connections have changed, and the Panel might have to be repainted
	 * @return true if changed
	 */
	public boolean getStatusChanged();
}
