package de.tu_darmstadt.tk.SmartHomeNetworkSim.view;

import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.JFrame;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Link;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation.SimpleLink;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.JComboBox;
import javax.swing.JButton;

@SuppressWarnings("serial")
public class LinkCreationPanel extends JScrollPane{
	
	JPanel content;
	private JTextField tfName;
	
	private Window frame;
	
	private Link newLink;
	private SmartDevice[] devices;
	private boolean edit;
	
	private Controller controller;
	public LinkCreationPanel(Link link, Controller controller) {
		this.controller = controller;
		newLink = link;
		this.devices = new SmartDevice[link.getDevices().size()];
		int i=0;
		for(SmartDevice d: link.getDevices()){
			this.devices[i++] = d;
		}
		edit = true;
		initializePanel();
	}
	
	public LinkCreationPanel(Collection<SmartDevice> devices, Controller controller) {
		this.controller = controller;
		newLink = new SimpleLink("LinkName");
		this.devices = new SmartDevice[devices.size()];
		int i=0;
		for(SmartDevice d: devices){
			controller.addLinkToDevice(newLink, d);
			this.devices[i++] = d;
		}
		edit = false;
		initializePanel();
	}
	
	private void initializePanel() {
		setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		this.setPreferredSize(new Dimension(600, 170 + devices.length*20));
		content = new JPanel();
		content.setPreferredSize(new Dimension(600, 150 + devices.length*20));
		this.setViewportView(content);
		content.setLayout(null);
		
		JLabel lblDescription = new JLabel("Create a new Link");
		lblDescription.setBounds(12, 13, 476, 16);
		content.add(lblDescription);
		
		JLabel lblName = new JLabel("Name:");
		lblName.setHorizontalAlignment(SwingConstants.RIGHT);
		lblName.setBounds(12, 42, 138, 16);
		content.add(lblName);
		
		tfName = new JTextField();
		tfName.setText(newLink.getName());
		tfName.setBounds(162, 39, 300, 22);
		content.add(tfName);
		tfName.setColumns(10);
		tfName.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				newLink.setName(tfName.getText());
			}
		});
		
		JLabel lblLinkType = new JLabel("Type:");
		lblLinkType.setHorizontalAlignment(SwingConstants.RIGHT);
		lblLinkType.setBounds(12, 71, 138, 16);
		content.add(lblLinkType);
		
		JComboBox<String> cmbLinkType = new JComboBox<String>();
		cmbLinkType.addItem("SimpleLink");
		cmbLinkType.addItem("testZigbee");
		cmbLinkType.setBounds(162, 68, 116, 22);
		content.add(cmbLinkType);
		cmbLinkType.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				System.out.println("WARNING: No  further Link Types implemented");
			}
		});
		
		JButton btnImportLink = new JButton("Import Link Type");
		btnImportLink.setBounds(290, 67, 144, 25);
		content.add(btnImportLink);
		btnImportLink.addActionListener(a->System.out.println("WARNING: No import yet"));
		
		JButton btnCreate = new JButton("Verify and Create");
		btnCreate.setBounds(121, 103, 206, 25);
		content.add(btnCreate);
		btnCreate.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				if(!edit){
					controller.addLink(newLink);
				}
				newLink.setName(tfName.getText());
				content.setVisible(false);
				setVisible(false);
				if(frame != null){
					frame.setVisible(false);
					frame.dispose();
				}
			}
		});
		
		int currentHeight = 150;
		for(int i = 0; i<devices.length;i++){
			int pos = i;
			JLabel lblDevice = new JLabel(devices[i].getName()+":");
			lblDevice.setHorizontalAlignment(SwingConstants.RIGHT);
			lblDevice.setBounds(10, currentHeight, 230, 18);
			content.add(lblDevice);
			
			JComboBox<String> cmbDev = new JComboBox<String>();
			cmbDev.addItem("Connected");
			cmbDev.addItem("Disconnected");
			cmbDev.setBounds(250, currentHeight, 240, 18);
			content.add(cmbDev);
			cmbDev.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					if(cmbDev.getSelectedIndex()==0 && !newLink.getDevices().contains(devices[pos])){
						controller.addLinkToDevice(newLink, devices[pos]);
					}else if(cmbDev.getSelectedIndex()==1&&newLink.getDevices().contains(devices[pos])){
						controller.removeLinkFromDevice(newLink, devices[pos]);
					}
				}
			});
			currentHeight += 20;
		}
	}

	
	 /**
     * Test the Panel
     */
    private static void testGUI() {
        JFrame frame = new JFrame("LinkCreation Panel");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
 
        LinkCreationPanel panel;
        
        LinkedList<SmartDevice> devicesOfLink = new LinkedList<SmartDevice>();
        for(int i = 0; i<5; i++)
        	devicesOfLink.add(new SmartDevice("Device" + i));
        panel = new LinkCreationPanel(devicesOfLink, new Controller(new Model()));
        /*
        Link testNet = new SimpleLink("Test Network");
        for(int i = 0; i<5; i++)
        	testNet.addDevice(new SmartDevice("Device" + i));
        panel = new LinkCreationPanel(testNet);
        */
        panel.setFrame(frame);
        frame.setContentPane(panel);
        frame.pack();
        frame.setVisible(true);
    }
    

 
    public static void main(String[] args) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                testGUI();
            }
        });
    }

	/**
	 * @return the frame
	 */
	public Window getFrame() {
		return frame;
	}

	/**
	 * @param frame the frame to set
	 */
	public void setFrame(Window frame) {
		this.frame = frame;
	}
}